--***************************************************************************
-- Copyright (c) 1994 - Ohio Board of Regents and
--                    - University of Cincinnati, Cincinnati, OH.
--                    - Sidhartha Mohanty,
--                    - Department of Electrical & Computer Engineering,
--                    - Oct, 1994.
--***************************************************************************
--
-- Title:               Design library for Queueing Model. Version 0.0
--
-- File:                [SUITE: vhdl/project/queue] queue_pkg_body.vhdl
-- Created:             Mon Sept 12 09:58:29 1994
-- Author:              Sidhartha Mohanty <smohanty@thor.ece.uc.edu>
--
-- Keywords:
-- LRM_version:         IEEE Std. P1076-1992/C
--
-- Objective:
--
-- Usage:
-- Analyze after queue_pkg.vhdl.
--
-- Algorithm:
-- (None)
--
-- References:
--
-- Result:
--
-- History:
--
-- Comments:
--
--***************************************************************************
use std.textio.all;

package body queue_pkg is

function resolve_arc (input_tokens : token_array) return token is
    variable out_tok_hand : token_handshake;
    variable out_token    : token := (0, ready, FALSE, now);
begin 
    out_tok_hand := ready;
    out_token    := (0, ready, FALSE, now);

    if input_tokens'length = 0 then
	    return out_token;
    else
    for I in input_tokens'range loop
	if input_tokens(i).handshake > out_token.handshake then
            out_token := input_tokens(i);          
        end if;
    end loop;
        return out_token;
    end if;

end resolve_arc; 

function resolve_signal (drivers : bool_array) return boolean is

begin

    assert drivers'length /= 0
     report "No drivers to token_avail ."
     severity Failure;

    assert drivers'length <= 2
     report "Token_avail signal must be shared by two elements"
     severity failure;

    IF drivers'length = 1 then
       return (drivers(1));
    elsif drivers(1) = FALSE then
       return (drivers(2));
    else
       return (drivers(1));
    end IF;

end resolve_signal;

function resolve_bit (drivers : bit_vector) return bit is

   variable in_bit : bit;
begin
    in_bit := '0';
 
    for I in drivers'range loop
     if drivers(i) = '1' then in_bit := '1';
      else return in_bit;
     end if;
    end loop;
    
    return in_bit;

end resolve_bit;

function resolve_num (args : num) return integer is

   variable j  : integer := 0;   
   variable k  : integer := 0;
begin
   
     for I in args'range loop
       j := j + args(i);
     end loop;
   return (j);
end; 


--set of overloaded functions to convert information to strings for
--printing purposes.

--convert token handshaking enumeration types.

  function stringof (handshake : token_handshake) return string is

  type token_handshake_array is 
      array (token_handshake) of string (1 to 9);
  constant token_handshake_strings : token_handshake_array := (
      "ready    ", 
      "sending  ", 
      "accepted ", 
      "not_ready"
  );

  variable i : integer;
  variable out_string : string (1 to 9);

  begin

      i := 1;
      out_string := token_handshake_strings(handshake);

      while i < 9 and out_string(i+1) /= ' ' loop
      	i := i + 1;
      end loop;

      return out_string(1 to i);

  end stringof;

  function stringof (in_number : integer) return string is

  variable line, reverse_line : string (1 to 10) := "          ";
  variable  i, j, wk_number : integer;
  type char_array is array (integer range 0 to 9) of character;
  variable digit_to_letter : char_array := ('0', '1', '2', '3', '4',
                                            '5', '6', '7', '8', '9');

  begin

      if in_number = 0 then 
      	return "0";
      end if;
      i := 1;
      wk_number := in_number;
      while wk_number > 0 loop
      	reverse_line(i) := digit_to_letter(wk_number mod 10);
  	wk_number := wk_number / 10;
  	i := i + 1;
      end loop;

      j := 1;
      while j < i loop
      	line(j) := reverse_line(i - j);
  	j := j + 1;
      end loop;
    
      return line(1 to j-1);

  end stringof;

  function stringof (token_time : time) return string is

  begin

      return stringof(token_time / 1 fs) & " fs";    --convert to integer and
  					      --then use integer fn

  end stringof;

  function stringof (dist : distrib) return string is

  type distrib_array is array (distrib) of string (1 to 9);
  constant distrib_strings : distrib_array := (
             "unif     ", 
             "replicate",
  	     "norm     ",
  	     "poiss    ",
             "erl      ",
  	     "fixed    ",
             "expo     ",
             "negex    ",
             "random   "
  );

  variable i : integer;
  variable out_string : string (1 to 9);

  begin

      i := 1;
      out_string := distrib_strings(dist);
  
      while i < 9 and out_string(i+1) /= ' ' loop
     	i := i + 1;
      end loop;
                                          
      return out_string(1 to i);

  end stringof;

  function stringof (name : source_name) return string is

  type name_array is array (source_name) of string (1 to 2);
  constant name_strings : name_array := (
             "S0", "S1", "S2", "S3", "S4", "S5", "S6", "S7", "S8", "S9", 
             "P1", "P2", "P3", "P4", "P5", "P6", "P7", "P8", "P9",
  	     "Q1", "Q2", "Q3", "Q4", "Q5", "J1", "J2", "J3", "J4", "J5",
  	     "F1", "F2", "F3", "F4", "F5"
  );

  variable i : integer;
  variable out_string : string (1 to 2);

  begin

      i := 1;
      out_string := name_strings(name);
  
      while i < 2 and out_string(i+1) /= ' ' loop
     	i := i + 1;
      end loop;
                                          
      return out_string(1 to i);

  end stringof;

  function stringof (comp : comp_type) return string is

  type comp_type_array is array (comp_type) of string (1 to 11);
  constant comp_type_strings : comp_type_array := (
       "source_unit", "join_unit  ", "fork_unit  ",
       "queue_unit ", "server_unit", "sink_unit  "
  );

  variable i : integer;
  variable out_string : string (1 to 11);

  begin

      i := 1;
      out_string := comp_type_strings(comp);

      while i < 11 and out_string(i+1) /= ' ' loop
      	i := i + 1;
      end loop;

      return out_string(1 to i);

  end stringof;

  procedure print_formatted_message (message : string) is

      variable message_line : line;
  begin

      write(message_line, message);
      writeline(std.textio.output, message_line);

  end print_formatted_message;

  procedure print_formatted_message (num1, num2 : real) is

      variable message_line : line;
  begin

      write(message_line, num1);
      write(message_line, string'(", "));
      write(message_line, num2);
      writeline(std.textio.output, message_line);

  end print_formatted_message;

  procedure print_formatted_message (num1 : integer; num2 : real) is

      variable message_line : line;
  begin

      write(message_line, num1);
      write(message_line, string'(", "));
      write(message_line, num2);
      writeline(std.textio.output, message_line);

  end print_formatted_message;
  
  procedure print_handshake_message (message1 : string; name1 : string;
  				     message2 : string; handshake : token_handshake; 
  				     message3 : string;
  				     integer_id : integer;
  				     message4 : string;
  				     T : time) is

  begin
  
    print_formatted_message (message1 & name1
  			     & message2
  		             & stringof(handshake)
                             & message3 
                             & stringof(integer_id)
  			     & message4 & stringof(T));

  end print_handshake_message;
				     
  procedure print_source_message (message1 : string; name : string;
  				  message2 : string; dist : distrib;
  				  message3 : string; 
  				  num1 : real; num2 : real) is

  begin

    print_formatted_message (message1 & name 
  			     & message2 
  		 	     & stringof(dist)  
                             & message3);

    print_formatted_message(num1, num2);

  end print_source_message;

  procedure print_source_message (message1 : string; name : string;
  				  message2 : string; dist : distrib;
  				  message3 : string; 
  				  num1 : integer; num2 : real) is

  begin

    print_formatted_message (message1 & name 
  			     & message2 
  		 	     & stringof(dist)  
                             & message3);

    print_formatted_message(num1, num2);

  end print_source_message;

  procedure print_formatted_message (message1 : string; T : time) is

      variable message_line : line;

  begin

      write(message_line, string'(message1));
      write(message_line, T);
      writeline(std.textio.output, message_line);

  end print_formatted_message;

  procedure print_token_message (message1 : string; message2 : string; 
  			         T : time; message3 : string;
  			         num : integer) is

      variable message_line : line;

  begin

  	print_formatted_message (message1 & message2 & stringof(T)
  			         & message3 & stringof(num));

  end print_token_message;

  procedure print_joinfork_message (message1 : string; name : string;
  	 			    message2 : string; message3 : string; 
  				    message4 : string; T : time) is

  begin

    print_formatted_message (message1 & name
  			     & message2 
  		 	     & message3 & message4  & stringof(T));

  end print_joinfork_message;

  procedure print_queue_message (message1 : string; name1 : string;
  			         message2 : string; integer_id : integer;
  			         message3 : string; T: time) is
  begin

    print_formatted_message (message1 & name1 
  			     & message2 & stringof(integer_id)
  		 	     & message3 & stringof(T));

  end print_queue_message;

  procedure print_server_message (message1 : string; name1 : string;
  			          message2 : string; integer_id : integer;
  			          message3 : string; T: time) is
  begin

    print_formatted_message ("Server " & name1 
  			     & message2 & stringof(integer_id)
  			     & " at time: " & stringof(T));

  end print_server_message;

  procedure print_sink_message (message1 : string; name1 : string;
  			        message2 : string; integer_id : integer;
  			        message3 : string; T: time) is
  begin

    print_formatted_message ("Sink " & name1 
  			     & message2 & stringof(integer_id)
  			     & " at time: " & stringof(T));

  end print_sink_message;

end queue_pkg;
