--***************************************************************************
-- Copyright (c) 1994 - Ohio Board of Regents and
--                    - University of Cincinnati, Cincinnati, OH.
--                    - Sidhartha Mohanty,
--                    - Department of Electrical & Computer Engineering,
--                    - Sept, 1994.
--***************************************************************************
-- 
-- Title:               Design library for Queueing Model. Version 0.0.
-- 
-- File:                [SUITE: vhdl/project/queue] sink.vhdl
-- Created:             Sat Sept 17, 12:16:15 1994
-- Author:              Sidhartha Mohanty <smohanty@thor.ece.uc.edu>
-- 
-- Keywords:
-- LRM_version:         IEEE Std. P1076-1992/C
-- 
-- Objective:
--
-- Usage:
-- Analyze after queue packages and random package.
--
-- Algorithm:               
-- (None)
-- 
-- References:
-- 
-- Result:
-- 
-- History:
-- 
-- Comments:
-- 
--***************************************************************************

library queue;
use queue.queue_pkg.all;
use std.textio.all;

entity sink is

  generic (name          : string  := "S3";
           debug_control : debug   := none;
           file_name     : string  := "";
	   report_freq   : time    := 10 fs);

  port (in_arc  : inout arc bus);

end;

architecture behavior of sink is

  signal id : integer := 1;
  file outfile : text is out file_name;
  
begin
  
initialize : process

   variable integer_line : line;
   variable integer_id   : integer := 1;

  begin

    in_arc <= null;

    assert not std.textio.endfile(std.textio.input)
	report "(sink.vhdl) Number of entities in Queueing Model exceed number of integer ids. Increase the number."
	severity failure;

    readline(std.textio.input, integer_line);

    assert integer_line'length >= 0
	report "(sink.vhdl) Missing integer in identifiers file."
	severity failure;

    read(integer_line, integer_id);
    id <= integer_id;

    wait for 0 fs;

    wait;

end process;

sink : process

  variable number_tokens_consumed : integer := 0;
  variable mesg : line;

 begin

    in_arc <= null;
    loop
       wait on in_arc until (in_arc.handshake = sending);
       if (debug_control = handshaking or 
	   debug_control = summary) then
       print_handshake_message ("Sink: ", name, " received handshake signal: ", 
				 in_arc.handshake,
                                " from component: ", in_arc.id,
				" at Time: ", now);
       end if;
       if (in_arc.tok_avail = TRUE ) then
         assert FALSE
           report "Consuming tokens ****************************"
           severity note;
	 if (debug_control = trace_tokens or 
	     debug_control = summary) then
	 print_sink_message ("Sink: ", name, " consuming token from component: ",
                              in_arc.id, " at Time: ", now); 
	 end if;
         number_tokens_consumed := number_tokens_consumed + 1;
	 write(mesg, string'("Total tokens consumed by Sink "));
	 write(mesg, name);
	 write(mesg, string'(" at Time: "));
	 write(mesg, now);
	 write(mesg, string'(" = "));
	 write(mesg, number_tokens_consumed);
	 writeline(outfile, mesg);      
         print_token_message("Total tokens consumed by Sink ", 
			     " at Time: ", now, " = ", number_tokens_consumed);
         wait for 3 fs;
         in_arc <= (id, accepted, FALSE, now);
         wait for 0 fs;
	 if (debug_control = handshaking or 
	     debug_control = summary) then
	 print_handshake_message ("Sink: ", name, " sent handshake signal: ", 
		        	   in_arc.handshake,
                                  " to component: ", in_arc.id,
			          " at Time: ", now);
	 end if;
	 in_arc <= null;
	 wait for 0 fs;
       else
         wait for 1 fs;
         assert FALSE
           report "Waiting to consume tokens"
           severity note;
       end if;
     end loop;
 end process;

end behavior;
