
--Copyright (c) 1994 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.

--author: sidhartha mohanty.
--smohanty@thor.ece.uc.edu.
--------------------------------------------------------------------
--
-- Test file instantiating the following Queuing Model using the VHDL
-- design entities and package in this directory.
--
---------------------------------------------------------------------
-- rand.v -->S1	   S2<-- rand.v
--       (source) (source)
--          |      |
--     arc1 |	   | arc2
--	    |      |
--        +-+------+-+
--        |    J1    |
--        |  (join)  |
--        +-----+----+
-- 		| 
--		| arc3
--         +----+---+
--         |   F1   |
--         | (fork) |
--         +-+-+--+-+
--           | |  | 
--           | |  |
--        |--  +-| +-----|
--	+----+ +----+ +----+ 
--	| Q1 | | Q2 | | Q3 | (Queues)
--      +-+--+ +-+--+ +-+--+         
--        +--+   | +--- | 
--	     |   | |	
--         +-+---+-+-+
--         |    J2   |
--         |  (join) |
--         +----+----+
--		| arc10
--		|
--         +----+---+
--         |   F2   |
--         | (fork) |
--         +-+-+-+--+
--	     | | |
--    +------+ | +------+
--    | arc11  | arc12  | arc13
--    |        |        |
--    S3       S4       S5 <--- rand.v
--  (server) (server) (server)
--    |        | arc15  |
--    +------+ | +------+
--     arc14 | | | arc16
--         +-+-+-+-+
--         |  J3   |
--         |(join) |
--         +-+-+-+-+
--	       |
--	       |
--   	       S6
--------------------------------------------------------------------

library queue;
use queue.queue_pkg.all;
use std.textio.all;

entity tester is
  
end tester;

architecture test of tester is

  signal arc1, arc2, arc3, arc4, arc5, arc6,
         arc7, arc8, arc9, arc10, arc11,
	 arc12, arc13, arc14, arc15, arc16,
	 arc17 : arc bus;


  component source1
    generic (out_control   : distrib;   seed : integer;
             generation    : frequency; name : string;
             debug_control : debug := none;
	     report_freq   : time := 10 fs);
             
    port (out_arc : inout arc bus);
      
  end component;

  component join1
    generic (number_in     : integer;
             name          : string;
             debug_control : debug := none;
	     report_freq   : time := 10 fs);
             
    port (in_arc  : inout arc_array (1 to number_in) bus;
          out_arc : inout arc bus);
      
  end component;

  component queue1
    generic (queue_length   : integer; name : string;
             discipline     : queue_discipline; 
             debug_control  : debug := none;
	     report_freq    : time := 10 fs);
             
    port (in_arc : inout arc bus; out_arc : inout arc bus);
      
  end component;

  component fork1
    generic (out_control    : distrib;
             number_out     : integer;
             name           : string;
             seed           : integer;
             generation     : frequency;
             debug_control  : debug := none;
	     report_freq    : time := 10 fs);
             
    port (in_arc  : inout arc bus;
          out_arc : inout arc_array(1 to number_out) bus);
      
  end component;

  component server1
    generic (service_time     : time; 
	     name             : string;
             service_dist     : distrib;
             server_type      : service_type := single_token;
	     service_capacity : integer      := 1;
	     seed             : integer;
	     generation       : frequency; 
	     debug_control    : debug := none;
	     report_freq      : time := 10 fs);
             
    port (in_arc : inout arc bus; out_arc : inout arc bus);
      
  end component;

  component sink1
    generic (name          : string;
             debug_control : debug := none;
	     report_freq   : time := 10 fs);
             
    port (in_arc  : inout arc bus);
      
  end component;


  for SO1 : source1 use entity queue.source (behavior);
  for SO2 : source1 use entity queue.source (behavior);
  for JO1 : join1   use entity queue.join   (behavior);
  for JO2 : join1   use entity queue.join   (behavior);
  for JO3 : join1   use entity queue.join   (behavior);
  for QU1 : queue1  use entity queue.queue  (behavior);
  for QU2 : queue1  use entity queue.queue  (behavior);
  for QU3 : queue1  use entity queue.queue  (behavior);
  for FO1 : fork1   use entity queue.fork   (behavior);
  for FO2 : fork1   use entity queue.fork   (behavior);
  for SE3 : server1 use entity queue.service(behavior);
  for SE4 : server1 use entity queue.service(behavior);
  for SE5 : server1 use entity queue.service(behavior);
  for SI6 : sink1   use entity queue.sink   (behavior);

BEGIN  --  test 
  
   SO1 : source1
	generic map (unif, 3, (1.0, 100.0), "S1")
	port map (arc1);
   SO2 : source1
	generic map (unif, 5, (10.0, 20.0), "S2")
	port map (arc2);
   JO1 : join1
	generic map (2, "J1")
	port map (in_arc(1)=>arc1, in_arc(2)=>arc2, out_arc=>arc3);
   FO1 : fork1
	generic map (unif, 3, "F1", 0, (0.0001, 1.0))
	port map (arc3, out_arc(1)=>arc4, out_arc(2)=>arc5, out_arc(3)=>arc6);
   QU1 : queue1
	generic map (100000, "Q1", LIFO)
	port map (arc4, arc7);
   QU2 : queue1
	generic map (5000, "Q2", FIFO)
	port map (arc5, arc8);
   QU3 : queue1
	generic map (300, "Q3", FIFO)
	port map (arc6, arc9);
   JO2 : join1
	generic map (3, "J2")
	port map (in_arc(1)=>arc7, in_arc(2)=>arc8, in_arc(3)=>arc9, out_arc=>arc10);
   FO2 : fork1
	generic map (negex, 3, "F2", 1, (0.2, 0.5))
	port map (arc10, out_arc(1)=>arc11, out_arc(2)=>arc12, out_arc(3)=>arc13);
   SE3 : server1
	generic map (1 fs, "S3", unif, single_token, 1, 5, (20.0, 30.0))
	port map (arc11, arc14);
   SE4 : server1
	generic map (1 fs, "S4", poiss, multi_token, 5, 6, (15.0, 0.1))
	port map (arc12, arc15);
   SE5 : server1
	generic map (15 fs, "S5", erl, multi_token, 3, 8, (18.0, 27.0) )
	port map (arc13, arc16);
   JO3 : join1
	generic map (3, "J3")
	port map (in_arc(1)=>arc14, in_arc(2)=>arc15, in_arc(3)=>arc16,
                  out_arc=>arc17);
   SI6 : sink1
	generic map ("S6")
	port map (arc17);

end test;
