--***************************************************************************
-- Copyright (c) 1994 - Ohio Board of Regents and
--                    - University of Cincinnati, Cincinnati, OH.
--                    - Sidhartha Mohanty,
--                    - Department of Electrical & Computer Engineering,
--                    - Jan, 1994.
--***************************************************************************
--
-- Title:               Design library for Queueing Model. Version 0.0.
--
-- File:                [SUITE: vhdl/project/queue] test.v
-- Created:             Thu Jan 27 19:19:26 1994
-- Author:              Sidhartha Mohanty <smohanty@thor>
--
-- Keywords:
-- LRM_version:         IEEE Std. P1076-1992/C
--
-- Objective:
--
-- Usage: Test-bench for the Queuing Model. Analyze last.
-- 
-- Algorithm:
-- (None)
--
-- References:
--
-- Result:
--
-- History:
--
-- Comments:
--
--***************************************************************************
--
-- Test file instantiating the following Queuing Model using the VHDL
-- design entities and package in this directory.
--
--------------------------------------------------------------------
--	      S1 <---- rand.v	  
--          (source) 
--             |
--	       | arc1
--         +---+---+
--         |   J1  |
--         | (join)|
--         +---+---+
-- 	       |
--	       | arc2
--	 +-----+------+
--       |	      |
--       |     Q1     |
--	 |  (queue)   |
--	 |	      |
--       +-----+------+
--	       | arc3
--	       |
--         +---+---+
--         |   F1  |
--         | (fork)|
--         +---+---+
--	       | 
--             | arc4
--            S3 
--         (server)
--             | 
--    	       | arc5
--             | 
--         +-+-+-+-+
--         |  J2   |
--         |(join) |
--         +---+---+
--	       | arc6
--	       |
--             S6
--	     (sink)
--------------------------------------------------------------------

library WORK;
use work.queue_pkg.all;
use std.textio.all;

entity tester is
  
end tester;

architecture test of tester is

  signal arc1, arc2, arc3, arc4, arc5, arc6 : arc bus;
         

  component source1
    generic (out_control   : distrib;   seed : integer;
             generation    : frequency; name : string; file_name : string;
             debug_control : debug := summary; 
             report_freq   : time := 10 fs);
             
    port (out_arc : inout arc);
      
  end component;

  component join1
    generic (number_in     : integer;
             name          : string;
             file_name     : string;
             debug_control : debug := none;
             report_freq   : time := 10 fs);
             
    port (in_arc  : inout arc_array (1 to number_in) bus;
          out_arc : inout arc bus);
      
  end component;

  component queue1
    generic (queue_length  : integer;
             no_initial_tokens : integer;
             name          : string;
	     discipline	   : queue_discipline;
             file_name     : string;
             debug_control : debug := none;
             report_freq   : time := 10 fs);
             
    port (in_arc : inout arc bus; out_arc : inout arc bus);
      
  end component;

  component fork1
      generic (out_control   : distrib;
               number_out    : integer;
               name          : string;
               seed          : integer;
               generation    : frequency;
               file_name     : string;
               debug_control : debug := none;
               report_freq   : time := 10 fs);
             
      port (in_arc  : inout arc bus;
            out_arc : inout arc_array(1 to number_out) bus);
      
  end component;

  component server1
    generic (service_time  : time;
             name          : string;
	     service_dist  : distrib;
             server_type   : service_type := single_token;
	     service_capacity : integer      := 1;
             seed          : integer;
	     generation    : frequency;
             file_name     : string;
             debug_control : debug := none;
	     report_freq   : time := 10 fs);
             
    port (in_arc : inout arc bus; out_arc : inout arc bus);
      
  end component;

  component sink1
    generic (name          : string;
             file_name     : string;
             debug_control : debug := none;
	     report_freq   : time := 10 fs);
             
    port (in_arc  : inout arc bus);
      
  end component;


  for SO1 : source1 use entity WORK.source (behavior);
  for JO1 : join1   use entity WORK.join   (behavior);
  for QU1 : queue1  use entity WORK.queue  (behavior);
  for FO1 : fork1   use entity WORK.fork   (behavior);
  for SE3 : server1 use entity WORK.service(behavior);
  for JO2 : join1   use entity WORK.join   (behavior);
  for SI6 : sink1   use entity WORK.sink   (behavior);

BEGIN  --  test 
  
   SO1 : source1
	generic map (unif, 0, (10.0, 40.0), "S1", "results/source.out")
	port map (arc1);
   JO1 : join1
	generic map (1, "J1", "results/join1.out", summary)
	port map (in_arc(1)=>arc1, out_arc=>arc2);
   QU1 : queue1
	generic map (100000, 10, "Q1", FIFO, "results/queue.out", summary)
	port map (arc2, arc3);
   FO1 : fork1
	generic map (replicate, 1, "F1", 0, (0.1, 0.1), "results/fork.out")
	port map (arc3, out_arc(1)=>arc4);
   SE3 : server1
	generic map (10 fs, "S3", unif, multi_token, 2, 4, (10.0, 15.0),
                        "results/server.out", summary)
	port map (arc4, arc5);
   JO2 : join1
	generic map (1, "J2", "results/join2.out", summary)
	port map (in_arc(1)=>arc5,
                  out_arc=>arc6);
   SI6 : sink1
	generic map ("S6", "results/sink.out")
	port map (arc6);

end test;
